import { Mat } from '../types.ts';
import { ManagedFloat64Array } from './memory.ts';
/** A 2x2 matrix. */
export declare class Mat2 extends ManagedFloat64Array<4> implements Mat<2> {
    /** Return an identity Mat2. */
    static identity(): Mat2;
    /** Return a Mat2 from Mat3. */
    static fromMat3(m: Mat3): Mat2;
    private constructor();
    get length(): 4;
    add(rhs: Mat2): this;
    sub(rhs: Mat2): this;
    mul(rhs: Mat2): this;
    scale(factor: number): this;
    transpose(): this;
    invert(): boolean;
    det(): number;
}
/** A 3x3 matrix. */
export declare class Mat3 extends ManagedFloat64Array<9> implements Mat<3> {
    /** Return an identity Mat3. */
    static identity(): Mat3;
    /** Return a Mat3 from Mat2. */
    static fromMat2(m: Mat2): Mat3;
    /** Return a Mat3 from Mat4. */
    static fromMat4(m: Mat4): Mat3;
    private constructor();
    get length(): 9;
    add(rhs: Mat3): this;
    sub(rhs: Mat3): this;
    mul(rhs: Mat3): this;
    scale(factor: number): this;
    transpose(): this;
    /**
     * Inverts this Mat3.
     * @see https://en.wikipedia.org/wiki/Invertible_matrix#Inversion_of_3_%C3%97_3_matrices
     */
    invert(): boolean;
    det(): number;
    /** Coverts this to a normal matrix, which is the inverse transpose matrix. */
    normalMat(): boolean;
}
/** A 4x4 matrix. */
export declare class Mat4 extends ManagedFloat64Array<16> implements Mat<4> {
    /** Return an identity Mat4. */
    static identity(): Mat4;
    /** Return a Mat4 from Mat3. */
    static fromMat3(m: Mat3): Mat4;
    private constructor();
    get length(): 16;
    add(rhs: Mat4): this;
    sub(rhs: Mat4): this;
    mul(rhs: Mat4): this;
    scale(factor: number): this;
    transpose(): this;
    invert(): boolean;
    det(): number;
}
//# sourceMappingURL=mat.d.ts.map