import { Vec } from '../types.ts';
import { Mat2, Mat3, Mat4 } from './mat.ts';
import { ManagedFloat64Array } from './memory.ts';
/** A 2D vector. */
export declare class Vec2 extends ManagedFloat64Array<2> implements Vec<2> {
    constructor(x?: number, y?: number);
    get length(): 2;
    add(rhs: Vec2): this;
    sub(rhs: Vec2): this;
    mul(m: Mat2): this;
    /** Premultiplies 3x3 matrix to this. */
    mulMat3(m: Mat3): this;
    dot(rhs: Vec2): number;
    lerp(rhs: Vec2, t: number): this;
    scale(factor: number): this;
    normalize(): boolean;
}
/** A 3D vector. */
export declare class Vec3 extends ManagedFloat64Array<3> implements Vec<3> {
    constructor(x?: number, y?: number, z?: number);
    get length(): 3;
    add(rhs: Vec3): this;
    sub(rhs: Vec3): this;
    mul(m: Mat3): this;
    /** Premultiplies 4x4 matrix to this. */
    mulMat4(m: Mat4): this;
    /** Assign cross product of this with RHS to this. */
    cross(rhs: Vec3): this;
    dot(rhs: Vec3): number;
    lerp(rhs: Vec3, t: number): this;
    scale(factor: number): this;
    normalize(): boolean;
}
/** A 4D vector. */
export declare class Vec4 extends ManagedFloat64Array<4> implements Vec<4> {
    constructor(x?: number, y?: number, z?: number, w?: number);
    get length(): 4;
    add(rhs: Vec4): this;
    sub(rhs: Vec4): this;
    mul(m: Mat4): this;
    dot(rhs: Vec4): number;
    lerp(rhs: Vec4, t: number): this;
    scale(factor: number): this;
    normalize(): boolean;
}
//# sourceMappingURL=vec.d.ts.map